        program ctou96
c---------------------------------------------------------------------c
c                                                                     c
c      COMPUTER PROGRAMS IN SEISMOLOGY                                c
c      VOLUME XI                                                      c
c                                                                     c
c      PROGRAM: CTOU96                                               c
c                                                                     c
c      COPYRIGHT 2017                                                 c
c      R. B. Herrmann                                                 c
c      Department of Earth and Atmospheric Sciences                   c
c      Saint Louis University                                         c
c      221 North Grand Boulevard                                      c
c      St. Louis, Missouri 63013                                      c
c      U. S. A.                                                       c
c                                                                     c
c---------------------------------------------------------------------c
c       read the disperison results
c       and fit a bspline to the phase slowness
c       to estimate the group slowness and then compare the two
c
c       Since U = domega/dk thus  1/U = dk/d omega = d ( omega * 1/c) / domega
c       1/U = 1/c + omega * d (1/c)/d omega
c
c       To give a good fit to data plotted on a log period axis, we
c       fit the slowness as a function of ln f. The process is to

c       a) do a spline fit of 1/c vs ln f  and then 
c       b) predict 
c            1/U = 1/c +  d (1/c)/d( ln f)
c------
c       Changes:
c
c       Development:
c         The codes bfit, spline caome from David Russell's phase match filter
c         The codes bndacc, bndsol, h12 are from Lawson and Hanson
c         This was originally programmed to be very efficient, e.g.,
c         that the x values are ordered to be increasing. This makes the
c         portions of the code that must determnie the appropriate node
c         more efficient. However this requires extra care to sort things 
c
c         The comparison of the  observed y(i) slowness to the
c         predicted is used to create a residual weighting to select the best number of
c         breakpoints to use for an 1 fit to the data.
c
c         To make this more automated, perhaps try differenct numbers of break points
c         and find the one that gives best L1 norm fit. This would be another loop
c-----
        
        integer lun, ierr, ndsp,  NOBS
        parameter (NOBS=1000)
        common/dispersion/jlorr,jobs,jobsyn,jmode,fper,fobs,fobserr
        integer*4 jlorr(NOBS), jobs(NOBS), jobsyn(NOBS), jmode(NOBS)  
        real*4  fper(NOBS)   , fobs(NOBS)   , fobserr(NOBS)

        
c       arrays for phase velocity
        real*4  fcslo(NOBS), cslo(NOBS), cper(NOBS)
        real*4  fuslo(NOBS), uslo(NOBS), uper(NOBS)
        integer ncslo, nuslo
        real*4 yp(NOBS), dyp(NOBS), coef(NOBS)

        integer ilvry, nbpmx
        character dispin*40, dispout*40

        integer i

c------
c       read in phase and group velocity
c       compute frequency
c       compulte group and phase slowness
c       bfit the phase slowness and then
c       predict the group slowness

c------
        call gcmdln(ilvry, dispin, dispout,nbpmx)
        call rddisp(dispin,1,ierr,ndsp,NOBS,
     1      jlorr,jobs,jobsyn,jmode,
     2      fper,fobs,fobserr)
c-----
c       fill up the phase slowness array
c-----
c-----
c       jlorr() I*4 1 = Love, 2 = Rayleigh
c       jobs()  I*4 1 = phase velocity, 2=group velocity, 3=gamma
c       jobsyn() I*4    1 = observed, 2 synthetic
c       jmode() I*4 mode: 0=Fund, 1=1 st, etc
c       fper()  R*4 period
c       fobs()  R*4 Array of dispersion values 
c-----
c       work with phase velocities and get 
c       minimum and maxium period in phase velocity data set
c       we will only use group velocities in this range
c-----
        permin = 1.0e+38
        permax = 0
        ncslo = 0
        do i=1,ndsp
          if(jobs(i) .eq. 1 .and. jmode(i).eq.0
     1      .and. jlorr(i) .eq. ilvry)then
               ncslo = ncslo + 1
               fcslo(ncslo) = alog(1./fper(i))
               cper(ncslo) = fper(i)
               cslo(ncslo) = 1./fobs(i)
               if(fper(i).gt.permax)then
                        permax = fper(i)
               endif
               if(fper(i).lt.permin)then
                        permin = fper(i)
               endif
          endif
        enddo
c-----
c       now process the group velocities
c       For this code - only use group velocities in same
c        frequency band as phase velocity
c-----
        nuslo = 0
        do i=1,ndsp
          if(jobs(i) .eq. 2 .and. jmode(i).eq.0
     1      .and. jlorr(i) .eq. ilvry)then
               if(fper(i).ge.permin .and. fper(i).le.permax)then
                    nuslo = nuslo + 1
                    fuslo(nuslo) = alog(1./fper(i))
                    uslo(nuslo) = 1./fobs(i)
                    uper(ncslo) = fper(i)
               endif
          endif
        enddo
        WRITE(6,*)'Number of phase velocity slownesses:',ncslo
        WRITE(6,*)'Number of group velocity slownesses:',nuslo
        flmin = alog(1./permax)
        flmax = alog(1./permin)
        call process(ncslo,fcslo,cslo,cper,
     1     nuslo,fuslo,uslo,uper,flmin,flmax,
     1     nbpmx,coef,yp,dyp,dispout,ilvry,permin,permax)
        end

        subroutine process(nc,xc,yc,cp,nu,xu,yu,up,flmin,flmax,
     1     nbpmx,c,yp,dyp,dispout,ilvry,permin,permax)
        implicit none
c-----
c       nc - number of phase velocity values
c       xc - ln f for phase velocity 
c       yc - phase velocity
c       cp - phase velocity periods
c       nu - number of group velocity values
c       xu - lnf for group velocity 
c       yu - group velocity
c       up - group velocity periods
c       flmin - minimum frequency in phase slowness
c       flmax - maximum frequency in phase slowness
c       nbpmx - upper limit for L1 search for best number of breakpoints
c       c  - spline coeffficients 
c       yp - spline estimates for phase slowness
c       dyp - spline estimates for derive of phase slowness with ln f
c       dispout - name of output file
c       ilvry    1 Love
c                2 Rayleigh
c       permin   minimum period in observed phase velocities
c       permax   maximum period in observed phase velocties
c-----
        integer nc, nu
        real*4 xc(nc), yc(nc), xu(nu), yu(nu),cp(nc),up(nu)
        real*4 c(nc), yp(nc), dyp(nc)
        real flmin, flmax,permin,permax
        integer nbpmx
        character dispout*(*)
        integer ilvry

        integer NOBS
        parameter (NOBS=1000)
        common/dispersion/jlorr,jobs,jobsyn,jmode,fper,fobs,fobserr
        integer*4 jlorr(NOBS), jobs(NOBS), jobsyn(NOBS), jmode(NOBS)  
        real*4  fper(NOBS)   , fobs(NOBS)   , fobserr(NOBS)

        integer nbp

c-----
c       x - array of ln f
c       y - array of dispersion
c       z - array of flag +1 = phase, -1 = group
c       w - array of weights
c       t - array of periods
c-----
        real*4 x(NOBS), y(NOBS), z(NOBS), w(NOBS), t(NOBS), res(NOBS)
        integer i, m, nfd, mm, iter, itermx, nbpbest
        real cv, uv, uslo, per
        real sumn, sumx, sumxx, xmean, xsd, xsdbest
        real sloerrfloor
        integer nper
        real wn(NOBS), lnf(NOBS)
c-----
c       errslofloor is the acceptable error in slowness
c       for the L1 norm
c
c       If cvel=3.5 and acceptable error is 0.1 km/s, then
c           dc/c = - ds/s = - c ds
c       Thus  ds = - dc/c^2
c                = - 0.1/*3.5^2) = -0.1/12.25 = -0.008
c-----
        sloerrfloor = 0.008
        WRITE(6,*)'nc     :',nc
        WRITE(6,*)'nu     :',nu
        WRITE(6,*)'nbpmx  :',nbpmx
c       WRITE(6,*)'flmin  :',flmin
c       WRITE(6,*)'flmax  :',flmax
c-----
c       build the big data set
c       x(m) is the frequency measure
c       y(m) is the dispersion measure
c       z(m) = +1 for phase velocity
c              -1 for group velocity
c       w(m) weights, initially 1.0 but then
c              adjusted by residual
c       t(m) = periods - these are saved for nicer output, otherwise
c              the period would 1./exp(x(m)) which could have roundoff
c-----
        m = 0
        do i=1,nc
            m = m + 1
            x(m) = xc(i)
            y(m) = yc(i)
            z(m) = + 1
            w(m) = 1.0
            t(m) = cp(i)
        enddo
        do i=1,nu
            m = m + 1
            x(m) = xu(i)
            y(m) = yu(i)
            z(m) = -1
            w(m) = 1.0
            t(m) = up(i)
        enddo
c----------------------------------------------------
c-----
c       for the efficiency and necessity for loops in
c       bfit and spline5 order in terms of increasing x
c-----
        call sort5(x,y,z,t,w,m)

c-----
c       This is the main processing
c       First do a weighted least squares fit to spline model
c       Second evaluate y(x) and then dy/dx
c       Finally compare to observed data
c-----
        itermx = 5
        xsdbest=1.0e+30
        
        do nbp = 4,nbpmx
           do iter=1,itermx
              call spline4(x,y,z,w,c,m,nbp)
c-----
c             for the output we want the predicted c and U
c             for the original set
c------

c-----
c             nfd = 0 for spline
c                 = 1 for derivative
c-----
c-----
c             create a data set that matches observed completely
c             this will be the basis of statistics and weighting
c             Since the regression is over slowness, the residual must be
c               a residual in slowness
c             yp  is predicted phase slowness
c             dyp is the derivative of the phase slowness with respect
c                 to (ln f)
c-----
              nfd = 0
              call bfit(x,yp ,c,m,nfd)
              nfd = 1
              call bfit(x,dyp,c,m,nfd)
              mm = 0
              sumn = 0
              sumx = 0
              sumxx = 0
              do i=1,m
                  uslo = yp(i) + dyp(i)
                  uv = 1./uslo
                  cv = 1./yp(i)
                  per = 1./exp(x(i))
                  if(z(i).gt.0)then
c-----
c                      phase velocity
c-----
                      mm = mm + 1
                      res(mm) = y(i) - yp(i)
                   w(i) = sloerrfloor/amax1(abs(res(mm)),sloerrfloor)
c              WRITE(6,*)'C',i,per,1./y(i),cv,res(mm),w(i)
                  else if (z(i).lt.0.0)then
c-----
c                  output group velocity
c-----
                      mm = mm + 1
                      res(mm) = y(i) - uslo
                   w(i) = sloerrfloor/amax1(abs(res(mm)),sloerrfloor)
c              WRITE(6,*)'U',i,per,1./y(i),uv,res(mm),w(i)
                   endif
                   sumn  = sumn + 1
                   sumx  = sumx + res(mm)
                   sumxx = sumxx + res(mm)*res(mm)
                 enddo
               if(iter.eq.itermx)then
                 xmean = sumx/sumn
                 xsd = sqrt(abs(sumxx - sumn*xmean*xmean)/sumn)
                 write(6,*)'nbp=',nbp,'iter=',iter,
     1             ' mean=',xmean,' sd=',xsd
                 if(xsd.lt.xsdbest)then
                     xsdbest = xsd
                     nbpbest = nbp
                 endif
               endif
           enddo
        enddo
c-----
c       FINAL run now that the number of breakpoints is estimated
c-----
        WRITE(6,*)'Final run nbp=',nbpbest
           nbp = nbpbest
           do iter=1,itermx
              call spline4(x,y,z,w,c,m,nbp)
c-----
c             for the output we want the predicted c and U
c             for the original set
c------

c-----
c             nfd = 0 for spline
c                 = 1 for derivative
c-----
c-----
c             create a data set that matches observed completely
c             this will be the basis of statistics and weighting
c             Since the regression is over slowness, the residual must be
c               a residual in slowness
c-----
              nfd = 0
              call bfit(x,yp,c,m,nfd)
              nfd = 1
              call bfit(x,dyp,c,m,nfd)
              mm = 0
              sumn = 0
              sumx = 0
              sumxx = 0
              do i=1,m
                  uslo = yp(i) + dyp(i)
                  uv = 1./uslo
                  cv = 1./yp(i)
                  per = 1./exp(x(i))
                  if(z(i).gt.0)then
c-----
c                      phase velocity
c-----
                      mm = mm + 1
                      res(mm) = y(i) - yp(i)
                   w(i) = sloerrfloor/amax1(abs(res(mm)),sloerrfloor)
c              WRITE(6,*)'C',i,per,1./y(i),cv,res(mm),w(i)
                  else if (z(i).lt.0.0)then
c-----
c                  output group velocity
c-----
                      mm = mm + 1
                      res(mm) = y(i) - uslo
                   w(i) = sloerrfloor/amax1(abs(res(mm)),sloerrfloor)
c              WRITE(6,*)'U',i,per,1./y(i),uv,res(mm),w(i)
                   endif
                   sumn  = sumn + 1
                   sumx  = sumx + res(mm)
                   sumxx = sumxx + res(mm)*res(mm)
                 enddo
               if(iter.eq.itermx)then
                 xmean = sumx/sumn
                 xsd = sqrt(abs(sumxx - sumn*xmean*xmean)/sumn)
                 write(6,*)'nbp=',nbp,'iter=',iter,
     1             ' mean=',xmean,' sd=',xsd
                 if(xsd.lt.xsdbest)then
                     xsdbest = xsd
                     nbpbest = nbp
                 endif
               endif
           enddo
    
c-----
c       create a final predicted dispersion 
c-----
c-----
c       wn = array of incresing periods
c       NOBS - array dimension
c       permin - minimum period desired
c       permax - maximum period desired
c       nper - number of periods returned
c-----
        call getper(wn,NOBS,permin,permax,nper) 
C       WRITE(6,*)'wn(1)    :',wn(1)
C       WRITE(6,*)'wn(nper) :',wn(nper)
c-----
c       bfit requires everything in increasing x
c-----
        do i=1,nper
              lnf(nper+1-i) = alog(1./wn(i))
        enddo
C       WRITE(6,*)'lnf(1)   :',lnf(1)
C       WRITE(6,*)'lnf(nper):',lnf(nper)
        call  outevendisp(lnf,wn,c,nper,dispout,nbp,ilvry)
        return
        end 

        subroutine outevendisp(x,t,c,n,dispout,nbp,ilvry)
        integer n
        real x(n), t(n), c(n)
        character dispout*(*)
        integer nbp, ilvry

        integer NOBS
        parameter (NOBS=1000)
        real  yp(NOBS),dyp(NOBS)

        common/dispersion/jlorr,jobs,jobsyn,jmode,fper,fobs,fobserr
        integer*4 jlorr(NOBS), jobs(NOBS), jobsyn(NOBS), jmode(NOBS)  
        real*4  fper(NOBS)   , fobs(NOBS)   , fobserr(NOBS)

        real uslo, iv, cv, per
        integer i, mm, nfd

        nfd = 0
        call bfit(x,yp,c,n,nfd)
        nfd = 1
        call bfit(x,dyp,c,n,nfd)
c------
c      eventually the above will be an L1
c
c       1/U = 1/c + omega * d (1/c)/d omega
c-----
c       jlorr() I*4 1 = Love, 2 = Rayleigh
c       jobs()  I*4 1 = phase velocity, 2=group velocity, 3=gamma
c       jobsyn() I*4    1 = observed, 2 synthetic
c       jmode() I*4 mode: 0=Fund, 1=1 st, etc
c       fper()  R*4 period
c       fobs()  R*4 Array of dispersion values 
c-----
        mm = 0
        do i=1,n
          uslo = yp(i) + dyp(i)
          uv = 1./uslo
          cv = 1./yp(i)
          per = t(n+1-i)
c-----
c         output phase velocity
c-----
          mm = mm + 1
          jlorr(mm) = ilvry
          jobs(mm) = 1
          jobsyn(mm) = 2
          jmode(mm) = 0
          fper(mm) = per
          fobs(mm) = cv
          fobserr(mm) = 0.001
c-----
c         output group velocity
c-----
          mm = mm + 1
          jlorr(mm) = ilvry
          jobs(mm) = 2
          jobsyn(mm) = 2
          jmode(mm) = 0
          fper(mm) = per
          fobs(mm) = uv
          fobserr(mm) = 0.001
        enddo
        call wrdisp(dispout,1,mm,
     1      jlorr,jobs,jobsyn,jmode,
     2      fper,fobs,fobserr,NOBS)
          
c-----
        return
        end


        subroutine bfit(x,y,c,m,nfd)
c-----
c       use spline coefficients to define
c       fit to data
c
c       x   - array of abscissae
c       y   - array to be evaluated at x values
c       c   - array of spline coefficients
c       m   - number of points in x,y arrays
c       nfd - 0 evaluate spline
c           - 1 estimate derivative at point
c           - 2 estimate integral at point
c-----
        implicit none
        integer NP2
        parameter(NP2=2000)
        real x(m),y(m),c(m)
        integer m, nfd
        common/bsp/b(NP2),q(4),h
        real b, q, h
        real t
        real p1, p2, cp1, cp2, dp1, dp2
        real sum, suma, fa, fb, u, yfit
        integer ifl, jt, i, l, ic
c-----
c       FORTRAN FUNCTION DEFINITIONS
c-----
        p1(t)=.25*t**2*t
        p2(t)=-(1.-t)**2*(1.+t)*.75+1.
        cp1(t)=t**4/16.
        cp2(t)=t*(1./4.+t*(3./8.+t*(1./4.-3./16.*t)))
        dp1(t)=.75*t**2
        dp2(t)=.75+t*(1.5-2.25*t)
        sum=0.
        suma=0.
        ifl=0
        jt=1
        do 110 i=1,m
  80  if(x(i).le.b(jt+1)) go to 90
        if(nfd.lt.2) go to 85
        ifl=0
        fa=-(c(jt)+11.*c(jt+1))*h/16.
        fb=(11.*c(jt+2)+c(jt+3))*h/16.
        sum=sum+fb-fa
  85  jt=jt+1
        go to 80
  90  u=(x(i)-b(jt))/h
        if(nfd.ne.1 .and. nfd.ne.2)then
            q(1)=p1(1.-u)
            q(2)=p2(1.-u)
            q(3)=p2(u)
            q(4)=p1(u)
        elseif(nfd .eq. 1)then
            q(1)=-dp1(1.-u)/h
            q(2)=-dp2(1.-u)/h
            q(3)=dp2(u)/h
            q(4)=dp1(u)/h
        elseif(nfd .eq. 2)then
            if(ifl.ne.1) then
                ifl=1
                fa=-(c(jt)+11.*c(jt+1))*h/16.
                suma=sum-fa
            endif
            q(1)=-cp1(1.-u)*h
            q(2)=-cp2(1.-u)*h
            q(3)=cp2(u)*h
            q(4)=cp1(u)*h
        endif
        yfit=0.0
        do 100 l=1,4
            ic=jt-1+l
            yfit=yfit+c(ic)*q(l)
  100   continue
        y(i)=yfit+suma
  110   continue
        return
        end

        subroutine spline4(x,y,z,w,c,m,nbp)
        implicit none
        integer NP2
        parameter(NP2=2000)
c-----
c       x - abcissa
c       y - odinate
c       z - flag +1 = phase, -1 = group
c       w - weight
c-----
        real x(m),y(m),z(m),w(m),c(NP2)
        integer m, nbp

        common/bsp/b(NP2),q(4),h
        real b, q, h
        real g(NP2,5)
        real p1, p2, dp1, dp2, zero, u, t, rnorm
        integer mdg, nband, nc, ir, ip, i, jt, mt, ig
        real wt
c-----
c       FORTRAN FUNCTION DEFINITION
c-----
        p1(t)=.25*t**2*t
        p2(t)=-(1.-t)**2*(1.+t)*.75+1.
        dp1(t)=.75*t**2
        dp2(t)=.75+t*(1.5-2.25*t)
c-----
c       end FORTRAN FUNCTION DEFINITON
c-----
C       write(0,*)'spline:m,nbp',m,nbp
        zero=0.
        mdg=NP2
        nband=4
        if(nbp.gt.m-2) nbp=m-4
        if(nbp.lt.2) nbp=2
        nc=nbp+2
        b(1)=x(1)
        b(nbp)=x(m)
        h=(b(nbp)-b(1))/float(nbp-1)
        if(nbp.le.2) go to 30
        do 20 i=3,nbp
  20  b(i-1)=b(i-2)+h
  30    continue
        ir=1
        ip=1
        i=1
        jt=1
  40  mt=0
  50    continue
        if(x(i).gt.b(jt+1)) go to 60
        u=(x(i)-b(jt))/h
        ig=ir+mt

        if(z(i) .gt. 0.0)then
c-----
c            phase slowness
c-----
             g(ig,1)= w(i)*(p1(1.-u) )
             g(ig,2)= w(i)*(p2(1.-u) )
             g(ig,3)= w(i)*(p2(u)    )
             g(ig,4)= w(i)*(p1(u)    )
             g(ig,5)= w(i)*(y(i)     )
C            WRITE(6,*)'C: ',i,x(i),y(i),z(i)
         else
c-----
c            group slowness
c-----
             wt = 0.01
             g(ig,1)= w(i)*wt*(p1(1.-u) -dp1(1.-u)/h )
             g(ig,2)= w(i)*wt*(p2(1.-u) -dp2(1.-u)/h )
             g(ig,3)= w(i)*wt*(p2(u) -dp2(u)/h       )
             g(ig,4)= w(i)*wt*(p1(u) -dp1(u)/h       )
             g(ig,5)= w(i)*wt*(y(i)                  )
C            WRITE(6,*)'U: ',i,x(i),y(i),z(i)
         endif

        mt=mt+1
        if(i.eq.m) go to 60
        i=i+1
        go to 50
  60    continue
        call bndacc(g,mdg,nband,ip,ir,mt,jt)
        if(i.eq.m) go to 70
        jt=jt+1
        go to 40
  70    continue
        call bndsol(1,g,mdg,nband,ip,ir,c,nc,rnorm)
        return
        end

        subroutine bndacc(g,mdg,nb,ip,ir,mt,jt)
c-----
c       c.l. lawson and r.j. hanson, jet propulsion laboratory
c         sequential algorithm for banded least squares problem.
c       accumulation phase.   for solution phase use bndsol.
c
c       the calling program must set ir=1 and ip=1 before the
c         first call to bndacc for a new case.
c
c       the second subscript of g() must be dimensioned at least
c         nb+1 in the calling program.
c-----
        parameter(NP=2000,NP2=2000)
        dimension g(NP2,5)
        zero=0.
        nbp1=nb+1
        if(mt.le.0) return
        if(jt.eq.ip) go to 70
        if(jt.le.ir) go to 30
        do 10 i=1,mt
        ig1=jt+mt-i
        ig2=ir+mt-i
        do 10 j=1,nbp1
  10  g(ig1,j)=g(ig2,j)
        ie=jt-ir
        do 20 i=1,ie
        ig=ir+i-1
        do 20 j=1,nbp1
  20  g(ig,j)=zero
        ir=jt
  30  mu=min0(nb-1,ir-ip-1)
        if(mu.eq.0) go to 60
        do 50 l=1,mu
        k=min0(l,jt-ip)
        lp1=l+1
        ig=ip+l
        do 40 i=lp1,nb
        jg=i-k
  40  g(ig,jg)=g(ig,i)
        do 50 i=1,k
        jg=nbp1-i
  50  g(ig,jg)=zero
  60  ip=jt
  70  mh=ir+mt-ip
        kh=min0(nbp1,mh)
        do 80 i=1,kh
        call h12(1,i,max0(i+1,ir-ip+1),mh,g(ip,i),
     $           1,rho,g(ip,i+1),1,mdg,nbp1-i)
   80   continue
        ir=ip+kh
        if(kh.lt.nbp1) go to 100
        do 90 i=1,nb
  90  g(ir-1,i)=zero
 100    continue
        return
        end

        subroutine bndsol(mode,g,mdg,nb,ip,ir,x,n,rnorm)
c-----
c       c.l. lawson and r.j. hanson, jet propulsion laboratory
c       sequential solution of a banded least squares problem.
c       solution phase.  for the accumulation phase us bndacc.
c
c       the second subscript of g() must be dimensioned at
c       least nb+1 in the calling program
c-----
        parameter (LIN=5,LOT=6,LER=0)
        parameter(NP=2000,NP2=2000)
        dimension g(NP2,5),x(NP2)
        zero=0.
        rnorm=zero
        go to (10,90,50), mode
  10  do 20 j=1,n
  20  x(j)=g(j,nb+1)
        rsq=zero
        np1=n+1
        irm1=ir-1
        if(NP1.gt.irm1) go to 40
        do 30 j=np1,irm1
  30  rsq=rsq+g(j,nb+1)**2
        rnorm=sqrt(rsq)
  40    continue
  50  do 80 ii=1,n
        i=n+1-ii
        s=zero
        l=max0(0,i-ip)
        if(i.eq.n) go to 70
        ie=min0(n+1-i,nb)
        do 60 j=2,ie
        jg=j+l
        ix=i-1+j
  60  s=s+g(i,jg)*x(ix)
C  70  if(g(i,l+1)) 80,130,80
   70   continue
      if(g(i,l+1) .ne. 0.0) then
            go to 80
        else
            go to 130
        endif
  80  x(i)=(x(i)-s)/g(i,l+1)
        return
  90  do 120 j=1,n
        s=zero
        if(j.eq.1) go to 110
        i1=max0(1,j-nb+1)
        i2=j-1
        do 100 i=i1,i2
        l=j-i+1+max0(0,i-ip)
 100  s=s+x(i)*g(i,l)
 110  l=max0(0,j-ip)
C       if(g(j,l+1)) 120,130,120
        if(g(j,l+1) .ne. 0.0)then
            go to 120
        else
            go to 130
        endif
 120  x(j)=(x(j)-s)/g(j,l+1)
        return
 130    continue
c-----
c       cleanup
c-----
c-----
c       terminate program
c-----
        write(LER,*)'zero diagonal term in bndsol'
        stop
        end
        subroutine h12 (mode,lpivot,l1,m,u,iue,up,c,ice,icv,ncv)
c-----
c       construction and/or application of a single
c       householder transformation..     q = i+u*(u**t)/b
c
c       mode    = 1 or 2   to select algorithm h1 or h2.
c       lpivot is the index of the pivot element.
c       l1,m   if l1 .le. m   the transformation will be constructed to
c              zero elements indexed from l1 through m.   if l1 gt. m
c              the subroutine does an identity transformation.
c       u(),iuw,up    on entry to h1 u() contains the pivot vector.
c                   iue is the storage increment between elements.
c                                       on exit from h1 u() and up
c                   contain quantities defining the vector u of the
c                   householder transformation.   on entry to h2 u()
c                   and up should contain quantities previously computed
c                   by h1.  these will not be modified by h2.
c       c()    on entry to h1 or h2 c() contains a matrix which will be
c              regarded as a set of vectors to which the householder
c              transformation is to be applied.  on exit c() c
c               contains the set of transformed vectors.
c       ice    storage increment between elements of vectors in c().
c       icv    storage increment between vectors in c().
c       ncv    number of vectors in c() to be transformed. if ncv .le. 0
c              no operations will be done on c().
c-----
        parameter(NP=2000,NP2=2000)
        dimension u(iue,m),c(5*NP2)
        double precision sm,b
        one=1.
c
        if (0.ge.lpivot.or.lpivot.ge.l1.or.l1.gt.m) return
        cl=abs(u(1,lpivot))
        if (mode.eq.2) go to 60
c              ****** construct the transformation. ******
            do 10 j=l1,m
  10      cl=amax1(abs(u(1,j)),cl)
C       if (cl) 130,130,20
        if( cl .le. 0.0)then
            go to 130
        else
            go to 20
        endif
  20  clinv=one/cl
        sm=(dble(u(1,lpivot))*clinv)**2
            do 30 j=l1,m
  30      sm=sm+(dble(u(1,j))*clinv)**2
c               convert dble. prec. sm to sngl. prec. sm1
        sm1=sm
        cl=cl*sqrt(sm1)
C       if (u(1,lpivot)) 50,50,40
        if (u(1,lpivot) .le. 0.0) then
            go to 50
        else
            go to 40
        endif
  40  cl=-cl
  50  up=u(1,lpivot)-cl
        u(1,lpivot)=cl
        go to 70
c       ****** apply the transformation 1+u*(u**t)/b to c. ******
c
C  60  if (cl) 130,130,70
   60   continue
        if(cl .le. 0.0 )then
            go to 130
        else
            go to 70
        endif
  70  if (ncv.le.0) return
        b=dble(up)*u(1,lpivot)
c            b  must be nonpositive here.  if b=0., return.
c
C       if (b) 80,130,130
        if ( b .lt. 0.0 )then
            go to 80
        else
            go to 130
        endif
  80  b=one/b
        i2=1-icv+ice*(lpivot-1)
        incr=ice*(l1-lpivot)
            do 120 j=1,ncv
            i2=i2+icv
            i3=i2+incr
            i4=i3
            sm=c(i2)*dble(up)
                do 90 i=l1,m
                sm=sm+c(i3)*dble(u(1,i))
  90          i3=i3+ice
C          if (sm) 100,120,100
            if(sm .ne. 0.0)then
                go to 100
            else
                go to 120
            endif
 100      sm=sm*b
            c(i2)=c(i2)+sm*dble(up)
                do 110 i=l1,m
                c(i4)=c(i4)+sm*dble(u(1,i))
 110          i4=i4+ice
 120      continue
 130      return
            end

        subroutine uniq(x,y,m)
c-----
c       The x,y arrays is in order of increasing period in the x array
c       go through the arrays to eliminate any duplicate periods
c       so that the interpolation does not blow up
c-----
        implicit none
        integer m
        real x(m), y(m)
        integer i, mout, j
c-----
c       do a gruesome search with array replacement
c-----
        mout = m
        i = 1
 1000   continue
            i = i + 1
            if(i.gt.mout)go to 2000

            if(x(i) .eq. x(i-1))then
c-----
c           shift left
c-----
                mout = mout - 1
                do 3000 j=i,mout
                    x(j) = x(j+1)
                    y(j) = y(j+1)
 3000           continue
                i = i - 1
            endif
        go to 1000
 2000   continue
        m = mout
        return
        end

        subroutine sort5(x,y,z,t,w,n)
c-----
c       This provides a sort of items
c       x(i) is the array to be sorted
c       which has associated with it an y(i) array
c       no is the number of points to be sorted
c       After the sort the x(i),y(i) pairs will be ordered
c       from the least to the largest
c-----
c-----
c      Reference: http://en.wikipedia.org/wiki/Bubble_sort
c-----
       integer n
       real x(n), y(n), z(n), t(n), w(n)

       do i = n, 1, -1
           do j = 1 , i -1
               if(x(j) .gt. x(j+1))then
                   tmp = x(j)
                   x(j) = x(j+1)
                   x(j+1) = tmp

                   tmp = y(j)
                   y(j) = y(j+1)
                   y(j+1) = tmp

                   tmp = z(j)
                   z(j) = z(j+1)
                   z(j+1) = tmp

                   tmp = t(j)
                   t(j) = t(j+1)
                   t(j+1) = tmp

                   tmp = w(j)
                   w(j) = w(j+1)
                   w(j+1) = tmp
                endif
           enddo
       enddo
       return
       end

        subroutine sort(x,y,n)
c-----
c       This provides a sort of items
c       x(i) is the array to be sorted
c       which has associated with it an y(i) array
c       no is the number of points to be sorted
c       After the sort the x(i),y(i) pairs will be ordered
c       from the least to the largest
c-----
c-----
c      Reference: http://en.wikipedia.org/wiki/Bubble_sort
c-----
       integer n
       real x(n), y(n), z(n)

       do i = n, 1, -1
           do j = 1 , i -1
               if(x(j) .gt. x(j+1))then
                   tmp = x(j)
                   x(j) = x(j+1)
                   x(j+1) = tmp

                   tmp = y(j)
                   y(j) = y(j+1)
                   y(j+1) = tmp

                endif
           enddo
       enddo
       return
       end

        subroutine gcmdln(ilvry, dispin, dispout,nbpmx)
c-----
c      
c-----
        integer ilvry
        character dispin*(*), dispout*(*)
        integer nbp

        integer mnmarg

        character name*40

        integer i, nmarg
        ilvry = 0
        dispin = ''
        dispout = ''
        i = 0
        nbpmx = 4
        nmarg = mnmarg()
 1000   continue
           i = i + 1
           if(i.gt.nmarg)go to 2000
           call mgtarg(i,name)
           if(name(1:2) .eq. '-L')then
                 ilvry = 1
           else if(name(1:2) .eq. '-R')then
                 ilvry = 2
           else if(name(1:3) .eq. '-DI')then
                 i = i + 1
                 call mgtarg(i,dispin)
           else if(name(1:3) .eq. '-DO')then
                 i = i + 1
                 call mgtarg(i,dispout)
           else if(name(1:3) .eq. '-NB')then
                 i = i + 1
                 call mgtarg(i,name)
                 read(name,'(bn,i10)')nbpmx
           else if(name(1:2) .eq. '-?')then
                 call usage()
           else if(name(1:2) .eq. '-h')then
                 call usage()
           endif
        go to 1000
 2000   continue
c-----
c       safety
c-----
        WRITE(6,*)'nbp :',nbpmxmx
        
        if(nbpmx.lt.2) nbpmx = 4
        if(nbpmx.gt.20)nbpmx = 20
        if(dispin .eq. '')call usage ()
        if(ilvry.eq.0)call usage ()
        return 
        end

        subroutine usage()
        integer LER
        parameter(LER=0)

         WRITE(LER,*)'Usage: ctou -DI dispin -DO dispout -NBMAX nbreak',
     1   '  -L -R'
         WRITE(LER,*)
     1 ' -DI dispin  (required) Input dispersion file in surf96 format'
         WRITE(LER,*)
     1 ' -DO dispout (required) Output dispersion file in surf96 format'
         WRITE(LER,*)
     1 '                        containing least squares smooth phase '
         WRITE(LER,*)
     1 '                        and group velocity curves'
         WRITE(LER,*)
     1 '                        These are at selected periods between',
     2 ' the extremes in the data set and not at all observed points'
         WRITE(LER,*)
     1 ' -NBMAX  nbreak (default=4) Maximum number of breakpoints',
     1    ' for spline'
         WRITE(LER,*)
     1 ' -L          (no default) Process Love-wave dispersion'
         WRITE(LER,*)
     1 ' -R          (no default) Process Rayleigh-wave dispersion'
        write(LER,*)
     1 '   NOTE IT IS ASSUMED THAT ONLY THE FUNDAMENTAL MODE IS PRESENT'
        write(LER,*)
     1 '   NOTE The group slowness is included in the inversion, but'
        write(LER,*)
     1 '   NOTE is weighted 0.01 of phase slowness since the question'
        write(LER,*)
     1 '   NOTE is whether the group velocity is compatible with',
     2 ' the phase velocity'
        write(LER,*)
     1  '-?                   Display this usage message'
        write(LER,*)
     1  '-h                   Display this usage message'
         stop
        end

        function mnmarg()
c---------------------------------------------------------------------c
c                                                                     c
c      COMPUTER PROGRAMS IN SEISMOLOGY                                c
c      VOLUME V                                                       c
c                                                                     c
c      SUBROUTINE: MNMARG                                             c
c                                                                     c
c      COPYRIGHT 1996 R. B. Herrmann                                  c
c                                                                     c
c      Department of Earth and Atmospheric Sciences                   c
c      Saint Louis University                                         c
c      221 North Grand Boulevard                                      c
c      St. Louis, Missouri 63103                                      c
c      U. S. A.                                                       c
c                                                                     c
c---------------------------------------------------------------------c
        implicit none
        integer iargc
        integer mnmarg
            mnmarg = iargc()
        return
        end

        subroutine mgtarg(i,name)
c---------------------------------------------------------------------c
c                                                                     c
c      COMPUTER PROGRAMS IN SEISMOLOGY                                c
c      VOLUME V                                                       c
c                                                                     c
c      SUBROUTINE: MGTARG                                             c
c                                                                     c
c      COPYRIGHT 1996 R. B. Herrmann                                  c
c                                                                     c
c      Department of Earth and Atmospheric Sciences                   c
c      Saint Louis University                                         c
c      221 North Grand Boulevard                                      c
c      St. Louis, Missouri 63103                                      c
c      U. S. A.                                                       c
c                                                                     c
c---------------------------------------------------------------------c
c-----
c       return the i'th command line argument
c
c       This version works on SUN, IBM RS6000
c-----
        implicit none
        integer i
        character name*(*)
            call getarg(i,name)
        return
        end

        function lgstr(str)
c-----
c       function to find the length of a string
c       this will only be used with file system path names
c       thus the first blank 
c       indicates the end of the string
c-----
        implicit none
        character*(*) str
        integer*4 lgstr
        integer n, i
        n = len(str)
        lgstr = 1
        do 1000 i=n,1,-1
            lgstr = i
            if(str(i:i).ne.' ')goto 100
 1000   continue
  100   continue
        return
        end
 


        subroutine rddisp(fildsp,lun,ierr,ndsp,NOBS,
     1      jlorr,jobs,jobsyn,jmode,
     2      fper,fobs,fobserr)
c-----
c       CHANGES
c       19 JAN 2002 - Make obsyn=2 for synthetic, 1 for observed
c                     to be compatible with f96subs.f 2=syn/1=obs
c       05 MAR 2002 - synthetic denoted by 'T' and observed by 'X'
c-----
c       read a SURF96 dispersion file
c-----
c       fildsp  Ch* name of dispersion file
c       lun I*4 logical unit for read
c       ierr    I*4 Error code: -1 file does not exist
c       ndsp    I*4 number of dispersion values
c
c       ftype   A   should be SURF96
c       jlorr() I*4 1 = Love, 2 = Rayleigh
c       jobs()  I*4 1 = phase velocity, 2=group velocity, 3=gamma
c       jobsyn() I*4    1 = observed, 2 synthetic
c       jmode() I*4 mode: 0=Fund, 1=1 st, etc
c       fper()  R*4 period
c       fobs()  R*4 Array of ground velocities
c       fobserr()   R*4 Error in u()
c       
c
c
CSURF96 R U T 0  350.000     2.694     0.635  
c-----
c-----
c       command line variables
c-----
        character fildsp*(*)
        integer lun, ierr, ndsp,  NOBS
        integer*4 jlorr(NOBS), jobs(NOBS), jobsyn(NOBS), jmode(NOBS)  
        real*4  fper(NOBS)   , fobs(NOBS)   , fobserr(NOBS)
c-----
c       internal variables
c-----
        logical ext
        character instr*132
        character ic*1
        integer ilorr, imode, iobs
        ierr = -1
        ext = .false.
        inquire(file=fildsp,exist=ext)
        if(.not. ext)then
            ierr = -1
        else
            open(lun,file=fildsp,access='sequential',
     1      form='formatted',status='old')
            rewind lun
c-----
c           read values and parse them correctly
c-----
            ndsp = 0
 1000       continue
            read(lun,'(a)',end=9999)instr
            ls = lgstr(instr)
c-----
c           do the parsing
c-----
            if(instr(1:6).eq.'surf96' .or.
     1          instr(1:6).eq.'SURF96')then
c-----
c               now get wave type
c-----      
                lsep = 6
                call getblnk(instr,lsep,ls,lnobl)
                ic = instr(lnobl:lnobl)
                if(ic(1:1).eq.'R'. or. ic(1:1).eq.'r')then
                    ilorr = 2
                else if(ic(1:1).eq.'L'. or. ic(1:1).eq.'l')then
                    ilorr = 1
                else if(ic(1:1).eq.'A'. or. ic(1:1).eq.'a')then
                    ilorr = 0
                else
                    go to 1000
                endif
c-----
c               now get observation type
c-----
                lsep = lnobl
                call getblnk(instr,lsep,ls,lnobl)
                ic = instr(lnobl:lnobl)
                if(ic(1:1).eq.'C'. or. ic(1:1).eq.'c')then
                    iobs = 1
                else if(ic(1:1).eq.'U'. or. ic(1:1).eq.'u')then
                    iobs = 2
                else if(ic(1:1).eq.'G'. or. ic(1:1).eq.'g')then
                    iobs = 3
                else
                    go to 1000
                endif
c-----
c               now get whether observed or synthetic
c-----
                lsep = lnobl
                call getblnk(instr,lsep,ls,lnobl)
                ic = instr(lnobl:lnobl)
                if(ic(1:1).eq.'T'. or. ic(1:1).eq.'t')then
                    iobsyn = 2
c-----
c       the F designation is deprecated
c-----
                else if(ic(1:1).eq.'F'. or. ic(1:1).eq.'f')then
                    iobsyn = 1
                else if(ic(1:1).eq.'X'. or. ic(1:1).eq.'x')then
                    iobsyn = 1
                else
                    go to 1000
                endif
c-----
c-----
c               now get the values using list directed IO
c-----
                lsep = lnobl
                call getblnk(instr,lsep,ls,lnobl)
                read(instr(lnobl:ls),*)
     1          imode,per,obs,obserr
c-----
c       test for array dimensions
c-----
                if(ndsp.eq.NOBS)go to 9999
                ndsp = ndsp + 1
                jlorr(ndsp) = ilorr
                jobs(ndsp)  = iobs
                jobsyn(ndsp)  = iobsyn
                jmode(ndsp) = imode
                fper(ndsp)  = per
                fobs(ndsp)  = obs
                fobserr(ndsp)  = obserr
            endif
            
            go to 1000
        endif
 9999       continue
            close (lun)
            ierr = 0
        return
        end


        subroutine getblnk(instr,lsep,ls,lnobl)
c-----
c       determine first non-blank character
c
c       instr   Ch* Character string to be parsed
c       lsep    I*4 index of last non blank character
c       ls  I*4 length of input string
c       lnobl   I*4 index of first non blank character
c-----
        character instr*(*)
        integer lsep,ls,lnobl
        character tab*1
        tab=char(9)
        lnobl = lsep+1
        igotit = 0
        do 1000 i=lsep+1,ls
            if(igotit.eq.0)then
            if(instr(i:i).ne.' ' .and. instr(i:i).ne.tab)then
                lnobl = i
                igotit = 1
            endif
            endif
 1000   continue
        return
        end


        subroutine wrdisp(fildsp,lun,ndsp,
     1      jlorr,jobs,jobsyn,jmode,
     2      fper,fobs,fobserr,NOBS)
c-----
c       CHANGES
c       19 JAN 2002 - Make obsyn=2 for synthetic, 1 for observed
c                     to be compatible with f96subs.f 2=syn/1=obs
c       05 MAR 2002 - synthetic denoted by 'T' and observed by 'X'
c-----
c       read a SURF96 dispersion file
c-----
c       read a SURF96 dispersion file
c-----
c       fildsp  Ch* name of dispersion file
c       lun I*4 logical unit for read
c       ierr    I*4 Error code: -1 file does not exist
c       ndsp    I*4 number of dispersion values
c
c       ftype   A   should be SURF96
c       jlorr() I*4 1 = Love, 2 = Rayleigh
c       jobs()  I*4 1 = phase velocity, 2=group velocity, 3=gamma
c       jobsyn()I*4 1 = observed, 2 theoretical
c       jmode() I*4 mode: 0=Fund, 1=1'st, etc
c       fper()  R*4 period
c       fobs()  R*4 Array of ground velocities
c       fobserr()   R*4 Error in u()
c       
c
c
CSURF96 R U T -1   350.000     2.694     0.635  
CSURF96 R U F  0  290.0000   4.18800   1.27200  
c-----
c-----
c       command line variables
c-----
        character fildsp*(*)
        integer lun, ierr, ndsp,  NOBS
        integer*4 jlorr(NOBS), jobs(NOBS), jobsyn(NOBS), jmode(NOBS)  
        real*4  fper(NOBS)   , fobs(NOBS)   , fobserr(NOBS)
c-----
c       internal variables
c-----
        character ostr*12
        integer ilorr, imode, iobs
        logical ext
        ext = .false.
        inquire(file=fildsp,exist=ext)
        if(.not. ext)then
            open(lun,file=fildsp,access='sequential',
     1      form='formatted',status='new')
        else
            open(lun,file=fildsp,access='sequential',
     1      form='formatted',status='old')
        endif
            rewind lun
c-----
c           write values and parse them correctly
c-----
            do 1000 i=1,ndsp
                ostr= ' '
                ostr(1:6) = 'SURF96'
c-----
c               now set wave type
c-----      
                ilorr = jlorr(i) 
                iobs  = jobs(i) 
                iobsyn  = jobsyn(i) 
                mode  = jmode(i) 
                per    = fper(i)
                obs    = fobs(i)
                obserr = fobserr(i)
                if(ilorr.eq.0)then
                    ostr(7:8) = ' A'
                else if(ilorr.eq.1)then
                    ostr(7:8) = ' L'
                else if(ilorr.eq.2)then
                    ostr(7:8) = ' R'
                endif
c-----
c               now set observation type
c-----
                if(iobs.eq.1)then
                    ostr(9:10) = ' C'
                else if(iobs.eq.2)then
                    ostr(9:10) = ' U'
                else if(iobs.eq.3)then
                    ostr(9:10) = ' G'
                endif
c-----
c               now set observation type
c-----
                if(iobsyn.eq.2)then
                    ostr(11:12) = ' T'
                else if(iobsyn.eq.1)then
                    ostr(11:12) = ' X'
                endif
                write(lun,1)ostr,mode,per,obs,obserr
 1000       continue
    1   format(a12,1x,i3,1x,3g11.4)
        return
        end

        subroutine getper(wn,NX,pmin,pmax,nper) 
c-----
c       automatically create periods from the [pmin,pmax] limits
c       wn()    R*4 array of periods
c       NX  I   dimension of array
c       pmin    R*4 minimum period
c       pmax    R*4 maximum period
c       nper    I*4 number of periods generated
c----
        implicit none
        integer MAXFRQ
        parameter (MAXFRQ=100)
        integer NX, nper
        real wn(NX)
        real pmin, pmax

        real ymxlog, ymmin, tenpow, p
        integer iy, ii, nocy, mper, jj
        integer NP
        parameter (NP=40)
        real pfac(NP)
        data pfac/1.0, 1.1, 1.2, 1.3, 1.4, 1.5, 1.6, 1.7, 1.8, 1.9, 
     1      2.0, 2.1, 2.2, 2.3, 2.4, 2.5, 2.6, 2.7, 2.8, 2.9,
     2      3.0, 3.2, 3.4, 3.6, 3.8, 4.0, 4.2, 4.4, 4.6, 4.8,
     3      5.0, 5.5, 6.0, 6.5, 7.0, 7.5, 8.0, 8.5, 9.0, 9.5/

        nper = 0
c-----
c       determine starting power
c-----
        ymxlog = alog10(pmax)   
        ymmin  = alog10(pmin)   
        nocy = ymxlog - ymmin + 1 
        iy = ymmin
        if(ymmin .lt. 0)iy = iy - 1
        do 100 ii=iy,iy+nocy+2
            tenpow = 10.0**ii
            do 200 jj=1,NP
                p = pfac(jj)*tenpow
                if(p .ge. 0.99*pmin .and. p .le. 1.01*pmax)then
                    nper = nper + 1
                    mper = nper
                    wn(nper) = p
                    if(nper.eq.NX)go to 1000
                endif
 200        continue
 100    continue
 1000   continue
        return
        end
CCC   c-----
CCC   c             create a data set that matches observed completely
CCC   c             this will be the basis of statistics and weighting
CCC   c-----
CCC           nfd = 0
CCC           call bfit(x,yp,c,m,nfd)
CCC           nfd = 1
CCC           call bfit(x,dyp,c,m,nfd)
CCC           mm = 0
CCC           do i=1,m
CCC             uslo = yp(i) + dyp(i)
CCC             uv = 1./uslo
CCC             cv = 1./yp(i)
CCC             per = 1./exp(x(i))
CCC             if(z(i).gt.0)then
CCC   c-----
CCC   c         output phase velocity
CCC   c-----
CCC                  mm = mm + 1
CCC                  jlorr(mm) = ilvry
CCC                  jobs(mm) = 1
CCC                  jobsyn(mm) = 2
CCC                  jmode(mm) = 0
CCC                  fper(mm) = per
CCC                  fobs(mm) = cv
CCC                  fobserr(mm) = 0.001
CCC             else if (z(i).lt.0.0)then
CCC   c-----
CCC   c         output group velocity
CCC   c-----
CCC                  mm = mm + 1
CCC                  jlorr(mm) = ilvry
CCC                  jobs(mm) = 2
CCC                  jobsyn(mm) = 2
CCC                  jmode(mm) = 0
CCC                  fper(mm) = per
CCC                  fobs(mm) = uv
CCC                  fobserr(mm) = 0.001
CCC             endif
CCC           enddo
CCC           call wrdisp('N'//dispout,1,mm,
CCC        1      jlorr,jobs,jobsyn,jmode,
CCC        2      fper,fobs,fobserr,NOBS)
CCC   c-----
CCC   c       create a data set consisting of the original observations
CCC   c-----
CCC           mm = 0
CCC           do i=1,m
CCC             if(z(i).gt.0)then
CCC   c-----
CCC   c         output phase velocity
CCC   c-----
CCC                  mm = mm + 1
CCC                  jlorr(mm) = ilvry
CCC                  jobs(mm) = 1
CCC                  jobsyn(mm) = 2
CCC                  jmode(mm) = 0
CCC                  fper(mm) =  1./exp(x(i))
CCC                  fobs(mm) = 1./y(i)
CCC                  fobserr(mm) = 0.001
CCC             else if (z(i).lt.0.0)then
CCC   c-----
CCC   c         output group velocity
CCC   c-----
CCC                  mm = mm + 1
CCC                  jlorr(mm) = ilvry
CCC                  jobs(mm) = 2
CCC                  jobsyn(mm) = 2
CCC                  jmode(mm) = 0
CCC                  fper(mm) =  1./exp(x(i))
CCC                  fobs(mm) = 1./y(i)
CCC                  fobserr(mm) = 0.001
CCC             endif
CCC           enddo
CCC           call wrdisp('O'//dispout,1,mm,
CCC        1      jlorr,jobs,jobsyn,jmode,
CCC        2      fper,fobs,fobserr,NOBS)
